#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
import unittest

from rdflib import Literal, URIRef

from oc_ocdm.graph.graph_entity import GraphEntity
from oc_ocdm.graph.graph_set import GraphSet


class TestResponsibleAgent(unittest.TestCase):
    resp_agent = 'http://resp_agent.test/'

    @classmethod
    def setUpClass(cls) -> None:
        cls.graph_set = GraphSet("http://test/", "./info_dir/", "", False)

    def setUp(self):
        self.ra = self.graph_set.add_ra(self.resp_agent)
        self.ar = self.graph_set.add_ar(self.resp_agent)

    def test_has_name(self):
        name = "Name"
        result = self.ra.has_name(name)
        self.assertIsNone(result)

        triple = self.ra.res, GraphEntity.iri_name, Literal(name)
        self.assertIn(triple, self.ra.g)

    def test_has_given_name(self):
        given_name = "GivenName"
        result = self.ra.has_given_name(given_name)
        self.assertIsNone(result)

        triple = self.ra.res, GraphEntity.iri_given_name, Literal(given_name)
        self.assertIn(triple, self.ra.g)

    def test_has_family_name(self):
        family_name = "GivenName"
        result = self.ra.has_family_name(family_name)
        self.assertIsNone(result)

        triple = self.ra.res, GraphEntity.iri_family_name, Literal(family_name)
        self.assertIn(triple, self.ra.g)

    def test_has_related_agent(self):
        related_agent = URIRef("http://test/RelatedAgent")
        result = self.ra.has_related_agent(related_agent)
        self.assertIsNone(result)

        triple = self.ra.res, GraphEntity.iri_relation, related_agent
        self.assertIn(triple, self.ra.g)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2024, Arcangelo Massari <arcangelo.massari@unibo.it>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.

from typing import Dict, Optional, Tuple, Union

import redis
from tqdm import tqdm

from oc_ocdm.counter_handler.counter_handler import CounterHandler


class RedisCounterHandler(CounterHandler):
    """A concrete implementation of the ``CounterHandler`` interface that persistently stores
    the counter values within a Redis database."""

    def __init__(self, host: str = 'localhost', port: int = 6379, db: int = 0, password: Optional[str] = None) -> None:
        """
        Constructor of the ``RedisCounterHandler`` class.

        :param host: Redis server host
        :type host: str
        :param port: Redis server port
        :type port: int
        :param db: Redis database number
        :type db: int
        :param password: Redis password (if required)
        :type password: Optional[str]
        """
        self.redis = redis.Redis(host=host, port=port, db=db, password=password, decode_responses=True)

    def set_counter(self, new_value: int, entity_short_name: str, prov_short_name: str = "",
                    identifier: int = 1, supplier_prefix: str = "") -> None:
        """
        It allows to set the counter value of graph and provenance entities.

        :param new_value: The new counter value to be set
        :type new_value: int
        :param entity_short_name: The short name associated either to the type of the entity itself
         or, in case of a provenance entity, to the type of the relative graph entity.
        :type entity_short_name: str
        :param prov_short_name: In case of a provenance entity, the short name associated to the type
         of the entity itself. An empty string otherwise.
        :type prov_short_name: str
        :param identifier: In case of a provenance entity, the counter value that identifies the relative
          graph entity. The integer value '1' otherwise.
        :type identifier: int
        :param supplier_prefix: The supplier prefix
        :type supplier_prefix: str
        :raises ValueError: if ``new_value`` is a negative integer
        :return: None
        """
        if new_value < 0:
            raise ValueError("new_value must be a non negative integer!")

        key = self._get_key(entity_short_name, prov_short_name, identifier, supplier_prefix)
        self.redis.set(key, new_value)

    def read_counter(self, entity_short_name: str, prov_short_name: str = "", identifier: int = 1, supplier_prefix: str = "") -> int:
        """
        It allows to read the counter value of graph and provenance entities.

        :param entity_short_name: The short name associated either to the type of the entity itself
         or, in case of a provenance entity, to the type of the relative graph entity.
        :type entity_short_name: str
        :param prov_short_name: In case of a provenance entity, the short name associated to the type
         of the entity itself. An empty string otherwise.
        :type prov_short_name: str
        :param identifier: In case of a provenance entity, the counter value that identifies the relative
          graph entity. The integer value '1' otherwise.
        :type identifier: int
        :param supplier_prefix: The supplier prefix
        :type supplier_prefix: str
        :return: The requested counter value.
        """
        key = self._get_key(entity_short_name, prov_short_name, identifier, supplier_prefix)
        value = self.redis.get(key)
        return int(value) if value is not None else 0

    def increment_counter(self, entity_short_name: str, prov_short_name: str = "", identifier: int = 1, supplier_prefix: str = "") -> int:
        """
        It allows to increment the counter value of graph and provenance entities by one unit.

        :param entity_short_name: The short name associated either to the type of the entity itself
         or, in case of a provenance entity, to the type of the relative graph entity.
        :type entity_short_name: str
        :param prov_short_name: In case of a provenance entity, the short name associated to the type
         of the entity itself. An empty string otherwise.
        :type prov_short_name: str
        :param identifier: In case of a provenance entity, the counter value that identifies the relative
          graph entity. The integer value '1' otherwise.
        :type identifier: int
        :param supplier_prefix: The supplier prefix
        :type supplier_prefix: str
        :return: The newly-updated (already incremented) counter value.
        """
        key = self._get_key(entity_short_name, prov_short_name, identifier, supplier_prefix)
        return self.redis.incr(key)

    def set_metadata_counter(self, new_value: int, entity_short_name: str, dataset_name: str) -> None:
        """
        It allows to set the counter value of metadata entities.

        :param new_value: The new counter value to be set
        :type new_value: int
        :param entity_short_name: The short name associated either to the type of the entity itself.
        :type entity_short_name: str
        :param dataset_name: In case of a ``Dataset``, its name. Otherwise, the name of the relative dataset.
        :type dataset_name: str
        :raises ValueError: if ``new_value`` is a negative integer or ``dataset_name`` is None
        :return: None
        """
        if new_value < 0:
            raise ValueError("new_value must be a non negative integer!")

        if dataset_name is None:
            raise ValueError("dataset_name must be provided!")

        key = f"metadata:{dataset_name}:{entity_short_name}"
        self.redis.set(key, new_value)

    def read_metadata_counter(self, entity_short_name: str, dataset_name: str) -> int:
        """
        It allows to read the counter value of metadata entities.

        :param entity_short_name: The short name associated either to the type of the entity itself.
        :type entity_short_name: str
        :param dataset_name: In case of a ``Dataset``, its name. Otherwise, the name of the relative dataset.
        :type dataset_name: str
        :raises ValueError: if ``dataset_name`` is None
        :return: The requested counter value.
        """
        if dataset_name is None:
            raise ValueError("dataset_name must be provided!")

        key = f"metadata:{dataset_name}:{entity_short_name}"
        value = self.redis.get(key)
        return int(value) if value is not None else 0

    def increment_metadata_counter(self, entity_short_name: str, dataset_name: str) -> int:
        """
        It allows to increment the counter value of metadata entities by one unit.

        :param entity_short_name: The short name associated either to the type of the entity itself.
        :type entity_short_name: str
        :param dataset_name: In case of a ``Dataset``, its name. Otherwise, the name of the relative dataset.
        :type dataset_name: str
        :raises ValueError: if ``dataset_name`` is None
        :return: The newly-updated (already incremented) counter value.
        """
        if dataset_name is None:
            raise ValueError("dataset_name must be provided!")

        key = f"metadata:{dataset_name}:{entity_short_name}"
        return self.redis.incr(key)

    def _get_key(self, entity_short_name: str, prov_short_name: str = "", identifier: Union[str, int, None] = None, supplier_prefix: str = "") -> str:
        """
        Generate a Redis key for the given parameters.

        :param entity_short_name: The short name associated either to the type of the entity itself
         or, in case of a provenance entity, to the type of the relative graph entity.
        :type entity_short_name: str
        :param prov_short_name: In case of a provenance entity, the short name associated to the type
         of the entity itself. An empty string otherwise.
        :type prov_short_name: str
        :param identifier: In case of a provenance entity, the identifier of the relative graph entity.
        :type identifier: Union[str, int, None]
        :param supplier_prefix: The supplier prefix
        :type supplier_prefix: str
        :return: The generated Redis key
        :rtype: str
        """
        key_parts = [entity_short_name, supplier_prefix]
        if prov_short_name:
            key_parts.append(str(identifier))
            key_parts.append(prov_short_name)
        return ':'.join(filter(None, key_parts))

    def batch_update_counters(self, updates: Dict[str, Dict[Tuple[str, str], Dict[int, int]]]) -> None:
        """
        Perform batch updates of counters, processing 1 million at a time with a progress bar.

        :param updates: A dictionary structure containing the updates.
            The structure is as follows:
            {
                supplier_prefix: {
                    (short_name, prov_short_name): {
                        identifier: counter_value
                    }
                }
            }
        :type updates: Dict[str, Dict[Tuple[str, str], Dict[int, int]]]
        """
        all_updates = []
        for supplier_prefix, value in updates.items():
            for (short_name, prov_short_name), counters in value.items():
                for identifier, counter_value in counters.items():
                    key = self._get_key(short_name, prov_short_name, identifier, supplier_prefix)
                    all_updates.append((key, counter_value))
        
        total_updates = len(all_updates)
        batch_size = 1_000_000

        with tqdm(total=total_updates, desc="Updating counters") as pbar:
            for i in range(0, total_updates, batch_size):
                batch = all_updates[i:i+batch_size]
                pipeline = self.redis.pipeline()
                for key, value in batch:
                    pipeline.set(key, value)
                pipeline.execute()
                pbar.update(len(batch))


#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
import os
import unittest

from oc_ocdm.counter_handler.filesystem_counter_handler import FilesystemCounterHandler


class TestFilesystemCounterHandler(unittest.TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        cls.info_dir = './info_dir/'
        cls.counter_handler = FilesystemCounterHandler(cls.info_dir)

        cls.file_path = cls.info_dir + 'test_file.txt'
        if not os.path.exists(os.path.dirname(cls.file_path)):
            os.makedirs(os.path.dirname(cls.file_path))

    def setUp(self):
        # Reset test file content:
        with open(self.file_path, 'w') as file:
            file.write("\n")

    def test_set_number(self):
        number = 18
        num_of_line = 35
        self.counter_handler._set_number(number, self.file_path, num_of_line)
        read_number = self.counter_handler._read_number(self.file_path, num_of_line)
        self.assertEqual(read_number, number)
        self.assertRaises(ValueError, self.counter_handler._set_number, -1, self.file_path, 1)
        self.assertRaises(ValueError, self.counter_handler._set_number, 1, self.file_path, -1)

    def test_read_number(self):
        number = 18
        num_of_line = 35
        self.counter_handler._set_number(number, self.file_path, num_of_line)
        
        read_number = self.counter_handler._read_number(self.file_path, num_of_line)
        self.assertEqual(read_number, number)

        self.assertRaises(ValueError, self.counter_handler._read_number, self.file_path, -1)

    def test_add_number(self):
        number = 18
        num_of_line = 35
        self.counter_handler._set_number(number, self.file_path, num_of_line)

        read_number = self.counter_handler._add_number(self.file_path, num_of_line)
        self.assertEqual(read_number, number + 1)

        self.assertRaises(ValueError, self.counter_handler._add_number, self.file_path, -1)

    def test_set_counters_batch(self):
        updates = {("br", "se"): {1: 10, 2: 20, 3: 30}}
        self.counter_handler.set_counters_batch(updates, "")
        
        for line_number, expected_value in updates[("br", "se")].items():
            read_value = self.counter_handler.read_counter("br", "se", line_number)
            self.assertEqual(read_value, expected_value)

    def test_read_metadata_counter(self):
        dataset_name: str = "http://dataset/"
        self.assertRaises(ValueError, self.counter_handler.read_metadata_counter, "xyz", dataset_name)
        self.assertRaises(ValueError, self.counter_handler.read_metadata_counter, "di", None)

    def test_increment_metadata_counter(self):
        dataset_name: str = "http://dataset/"
        self.assertRaises(ValueError, self.counter_handler.increment_metadata_counter, "xyz", dataset_name)
        self.assertRaises(ValueError, self.counter_handler.increment_metadata_counter, "di", None)

    def test_set_metadata_counter(self):
        dataset_name = "http://dataset/"
        entity_short_name = "di"
        value = 42
        
        self.counter_handler.set_metadata_counter(value, entity_short_name, dataset_name)
        read_value = self.counter_handler.read_metadata_counter(entity_short_name, dataset_name)
        self.assertEqual(read_value, value)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from typing import List, Dict

from oc_ocdm.counter_handler.counter_handler import CounterHandler


class InMemoryCounterHandler(CounterHandler):
    """A concrete implementation of the ``CounterHandler`` interface that temporarily stores
    the counter values in the volatile system memory."""

    def __init__(self) -> None:
        """
        Constructor of the ``InMemoryCounterHandler`` class.
        """
        self.short_names: List[str] = ["an", "ar", "be", "br", "ci", "de", "id", "pl", "ra", "re", "rp"]
        self.prov_short_names: List[str] = ["se"]
        self.metadata_short_names: List[str] = ["di"]
        self.entity_counters: Dict[str, int] = {key: 0 for key in self.short_names}
        self.prov_counters: Dict[str, Dict[str, List[int]]] = {key1: {key2: [] for key2 in self.prov_short_names}
                                                               for key1 in self.short_names}
        self.metadata_counters: Dict[str, Dict[str, int]] = {}

    def set_counter(self, new_value: int, entity_short_name: str, prov_short_name: str = "",
                    identifier: int = 1, supplier_prefix: str = "") -> None:
        """
        It allows to set the counter value of graph and provenance entities.

        :param new_value: The new counter value to be set
        :type new_value: int
        :param entity_short_name: The short name associated either to the type of the entity itself
         or, in case of a provenance entity, to the type of the relative graph entity.
        :type entity_short_name: str
        :param prov_short_name: In case of a provenance entity, the short name associated to the type
         of the entity itself. An empty string otherwise.
        :type prov_short_name: str
        :param identifier: In case of a provenance entity, the counter value that identifies the relative
          graph entity. The integer value '1' otherwise.
        :type identifier: int
        :raises ValueError: if ``new_value`` is a negative integer, ``identifier`` is less than or equal to zero,
          ``entity_short_name`` is not a known short name or ``prov_short_name`` is not a known provenance short name.
        :return: None
        """
        if new_value < 0:
            raise ValueError("new_value must be a non negative integer!")

        if entity_short_name not in self.short_names:
            raise ValueError("entity_short_name is not a known short name!")

        if prov_short_name != "":
            if prov_short_name not in self.prov_short_names:
                raise ValueError("prov_short_name is not a known provenance short name!")
            if identifier <= 0:
                raise ValueError("identifier must be a positive non-zero integer number!")

        identifier -= 1  # Internally we use zero_indexing!
        if prov_short_name in self.prov_short_names:
            # It's a provenance entity!
            missing_counters: int = identifier - (len(self.prov_counters[entity_short_name][prov_short_name]) - 1)
            if missing_counters > 0:
                self.prov_counters[entity_short_name][prov_short_name] += [0] * missing_counters
            self.prov_counters[entity_short_name][prov_short_name][identifier] = new_value
        else:
            # It's an entity!
            self.entity_counters[entity_short_name] = new_value

    def read_counter(self, entity_short_name: str, prov_short_name: str = "", identifier: int = 1, supplier_prefix: str = "") -> int:
        """
        It allows to read the counter value of graph and provenance entities.

        :param entity_short_name: The short name associated either to the type of the entity itself
         or, in case of a provenance entity, to the type of the relative graph entity.
        :type entity_short_name: str
        :param prov_short_name: In case of a provenance entity, the short name associated to the type
         of the entity itself. An empty string otherwise.
        :type prov_short_name: str
        :param identifier: In case of a provenance entity, the counter value that identifies the relative
          graph entity. The integer value '1' otherwise.
        :type identifier: int
        :raises ValueError: if ``identifier`` is less than or equal to zero, ``entity_short_name``
          is not a known short name or ``prov_short_name`` is not a known provenance short name.
        :return: The requested counter value.
        """
        if entity_short_name not in self.short_names:
            raise ValueError("entity_short_name is not a known short name!")

        if prov_short_name != "":
            if prov_short_name not in self.prov_short_names:
                raise ValueError("prov_short_name is not a known provenance short name!")
            if identifier <= 0:
                raise ValueError("identifier must be a positive non-zero integer number!")

        identifier -= 1  # Internally we use zero_indexing!
        if prov_short_name in self.prov_short_names:
            # It's a provenance entity!
            missing_counters: int = identifier - (len(self.prov_counters[entity_short_name][prov_short_name]) - 1)
            if missing_counters > 0:
                self.prov_counters[entity_short_name][prov_short_name] += [0] * missing_counters
            return self.prov_counters[entity_short_name][prov_short_name][identifier]
        else:
            # It's an entity!
            return self.entity_counters[entity_short_name]

    def increment_counter(self, entity_short_name: str, prov_short_name: str = "", identifier: int = 1, supplier_prefix: str = "") -> int:
        """
        It allows to increment the counter value of graph and provenance entities by one unit.

        :param entity_short_name: The short name associated either to the type of the entity itself
         or, in case of a provenance entity, to the type of the relative graph entity.
        :type entity_short_name: str
        :param prov_short_name: In case of a provenance entity, the short name associated to the type
         of the entity itself. An empty string otherwise.
        :type prov_short_name: str
        :param identifier: In case of a provenance entity, the counter value that identifies the relative
          graph entity. The integer value '1' otherwise.
        :type identifier: int
        :raises ValueError: if ``identifier`` is less than or equal to zero, ``entity_short_name``
          is not a known short name or ``prov_short_name`` is not a known provenance short name.
        :return: The newly-updated (already incremented) counter value.
        """
        if entity_short_name not in self.short_names:
            raise ValueError("entity_short_name is not a known short name!")

        if prov_short_name != "":
            if prov_short_name not in self.prov_short_names:
                raise ValueError("prov_short_name is not a known provenance short name!")
            if identifier <= 0:
                raise ValueError("identifier must be a positive non-zero integer number!")

        identifier -= 1  # Internally we use zero_indexing!
        if prov_short_name in self.prov_short_names:
            # It's a provenance entity!
            missing_counters: int = identifier - (len(self.prov_counters[entity_short_name][prov_short_name]) - 1)
            if missing_counters > 0:
                self.prov_counters[entity_short_name][prov_short_name] += [0]*missing_counters
            self.prov_counters[entity_short_name][prov_short_name][identifier] += 1
            return self.prov_counters[entity_short_name][prov_short_name][identifier]
        else:
            # It's an entity!
            self.entity_counters[entity_short_name] += 1
            return self.entity_counters[entity_short_name]

    def set_metadata_counter(self, new_value: int, entity_short_name: str, dataset_name: str) -> None:
        """
        It allows to set the counter value of metadata entities.

        :param new_value: The new counter value to be set
        :type new_value: int
        :param entity_short_name: The short name associated either to the type of the entity itself.
        :type entity_short_name: str
        :param dataset_name: In case of a ``Dataset``, its name. Otherwise, the name of the relative dataset.
        :type dataset_name: str
        :raises ValueError: if ``new_value`` is a negative integer, ``dataset_name`` is None or
          ``entity_short_name`` is not a known metadata short name.
        :return: None
        """
        if new_value < 0:
            raise ValueError("new_value must be a non negative integer!")

        if dataset_name is None:
            raise ValueError("dataset_name must be provided!")

        if entity_short_name not in self.metadata_short_names:
            raise ValueError("entity_short_name is not a known metadata short name!")

        if dataset_name not in self.metadata_counters:
            self.metadata_counters[dataset_name] = {key: 0 for key in self.metadata_short_names}

        self.metadata_counters[dataset_name][entity_short_name] = new_value

    def read_metadata_counter(self, entity_short_name: str, dataset_name: str) -> int:
        """
        It allows to read the counter value of metadata entities.

        :param entity_short_name: The short name associated either to the type of the entity itself.
        :type entity_short_name: str
        :param dataset_name: In case of a ``Dataset``, its name. Otherwise, the name of the relative dataset.
        :type dataset_name: str
        :raises ValueError: if ``dataset_name`` is None or ``entity_short_name`` is not a known metadata short name.
        :return: The requested counter value.
        """
        if dataset_name is None:
            raise ValueError("dataset_name must be provided!")

        if entity_short_name not in self.metadata_short_names:
            raise ValueError("entity_short_name is not a known metadata short name!")

        if dataset_name not in self.metadata_counters:
            return 0
        else:
            if entity_short_name not in self.metadata_counters[dataset_name]:
                return 0
            else:
                return self.metadata_counters[dataset_name][entity_short_name]

    def increment_metadata_counter(self, entity_short_name: str, dataset_name: str) -> int:
        """
        It allows to increment the counter value of metadata entities by one unit.

        :param entity_short_name: The short name associated either to the type of the entity itself.
        :type entity_short_name: str
        :param dataset_name: In case of a ``Dataset``, its name. Otherwise, the name of the relative dataset.
        :type dataset_name: str
        :raises ValueError: if ``dataset_name`` is None or ``entity_short_name`` is not a known metadata short name.
        :return: The newly-updated (already incremented) counter value.
        """
        if dataset_name is None:
            raise ValueError("dataset_name must be provided!")

        if entity_short_name not in self.metadata_short_names:
            raise ValueError("entity_short_name is not a known metadata short name!")

        if dataset_name not in self.metadata_counters:
            self.metadata_counters[dataset_name] = {key: 0 for key in self.metadata_short_names}

        self.metadata_counters[dataset_name][entity_short_name] += 1
        return self.metadata_counters[dataset_name][entity_short_name]



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
from __future__ import annotations

from functools import wraps
from typing import TYPE_CHECKING

from oc_ocdm.abstract_entity import AbstractEntity

if TYPE_CHECKING:
    from typing import Callable, Any
from rdflib import URIRef


def accepts_only(param_type: str):
    """
    A decorator that can be applied to the entity methods such as setters and removers
    when they accept a parameter. It enforces the right parameter type by raising a
    ``TypeError`` when the parameter is not None but its type is not the expected one.

    The expected type can be expressed through a short string:

      * 'literal' for the ``str`` type;
      * 'thing' for the ``URIRef`` type (from ``rdflib``);
      * '_dataset_' for the ``Dataset`` entities;
      * the OCDM short name in case of any other entity (e.g. 'br' for ``BibliographicResource``).

    :param param_type: A short string representing the expected type
    :type param_type: str
    """
    def accepts_only_decorator(function: Callable):

        @wraps(function)
        def accepts_only_wrapper(self, param: Any = None, **kwargs):
            lowercase_type = param_type.lower()
            if param is None or \
                    (lowercase_type == 'literal' and isinstance(param, str)) or \
                    (lowercase_type == 'thing' and isinstance(param, URIRef)) or \
                    (isinstance(param, AbstractEntity) and param.short_name == lowercase_type):
                return function(self, param, **kwargs)
            else:
                raise TypeError('[%s.%s] Expected argument type: %s. Provided argument type: %s.' %
                                (self.__class__.__name__, function.__name__, lowercase_type, type(param).__name__))

        return accepts_only_wrapper
    return accepts_only_decorator


#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
from __future__ import annotations

from typing import TYPE_CHECKING, Tuple, List, Optional

from oc_ocdm.abstract_entity import AbstractEntity
from rdflib import URIRef, Namespace, Graph

if TYPE_CHECKING:
    from typing import ClassVar, Dict
    from oc_ocdm.metadata.metadata_set import MetadataSet


class MetadataEntity(AbstractEntity):
    DCTERMS = Namespace("http://purl.org/dc/terms/")
    DCAT = Namespace("http://www.w3.org/ns/dcat#")
    VOID = Namespace("http://rdfs.org/ns/void#")

    iri_dataset = DCAT.Dataset
    iri_datafile = DCAT.Distribution

    iri_title = DCTERMS["title"]
    iri_description = DCTERMS.description
    iri_issued = DCTERMS.issued
    iri_modified = DCTERMS.modified
    iri_keyword = DCAT.keyword
    iri_subject = DCAT.theme
    iri_landing_page = DCAT.landingPage
    iri_subset = VOID.subset
    iri_sparql_endpoint = VOID.sparqlEndpoint
    iri_distribution = DCAT.distribution
    iri_license = DCTERMS.license
    iri_download_url = DCAT.downloadURL
    iri_media_type = DCAT.mediaType
    iri_byte_size = DCAT.byte_size

    short_name_to_type_iri: ClassVar[Dict[str, URIRef]] = {
        '_dataset_': iri_dataset,
        'di': iri_datafile
    }

    def __init__(self, g: Graph, base_iri: str, dataset_name: str, m_set: MetadataSet,
                 res: URIRef = None, res_type: URIRef = None, resp_agent: str = None,
                 source: str = None, count: str = None, label: str = None, short_name: str = "",
                 preexisting_graph: Graph = None) -> None:
        super(MetadataEntity, self).__init__()
        self.g: Graph = g
        self.base_iri: str = base_iri
        self.dataset_name: str = dataset_name
        self.resp_agent: str = resp_agent
        self.source: str = source
        self.short_name: str = short_name
        self.m_set: MetadataSet = m_set
        self.preexisting_graph: Graph = Graph(identifier=g.identifier)
        self._merge_list: Tuple[MetadataEntity] = tuple()
        # FLAGS
        self._to_be_deleted: bool = False
        self._was_merged: bool = False

        # If res was not specified, create from scratch the URI reference for this entity,
        # otherwise use the provided one
        if res is None:
            base_res: str = self.base_iri + self.dataset_name
            if base_res[-1] != '/':
                base_res += '/'
            self.res = self._generate_new_res(count, base_res, short_name)
        else:
            self.res = res

        if m_set is not None:
            # If not already done, register this MetadataEntity instance inside the MetadataSet
            if self.res not in m_set.res_to_entity:
                m_set.res_to_entity[self.res] = self

        if preexisting_graph is not None:
            # Triples inside self.g are entirely replaced by triples from preexisting_graph.
            # This has maximum priority with respect to every other self.g initializations.
            # It's fundamental that the preexisting graph gets passed as an argument of the constructor:
            # allowing the user to set this value later through a method would mean that the user could
            # set the preexisting graph AFTER having modified self.g (which would not make sense).
            self.remove_every_triple()
            for p, o in preexisting_graph.predicate_objects(self.res):
                self.g.add((self.res, p, o))
                self.preexisting_graph.add((self.res, p, o))
        else:
            # Add mandatory information to the entity graph
            self._create_type(res_type)
            if label is not None:
                self.create_label(label)

    @staticmethod
    def _generate_new_res(count: str, base_res: str, short_name: str) -> URIRef:
        if short_name == '_dataset_':
            return URIRef(base_res)
        else:
            return URIRef(base_res + short_name + "/" + count)

    @property
    def to_be_deleted(self) -> bool:
        return self._to_be_deleted

    @property
    def was_merged(self) -> bool:
        return self._was_merged

    @property
    def merge_list(self) -> Tuple[MetadataEntity]:
        return self._merge_list

    def mark_as_to_be_deleted(self) -> None:
        # Here we must REMOVE triples pointing
        # to 'self' [THIS CANNOT BE UNDONE]:
        for res, entity in self.m_set.res_to_entity.items():
            triples_list: List[Tuple] = list(entity.g.triples((res, None, self.res)))
            for triple in triples_list:
                entity.g.remove(triple)

        self._to_be_deleted = True

    def merge(self, other: MetadataEntity) -> None:
        """
        **WARNING:** ``MetadataEntity`` **is an abstract class that cannot be instantiated at runtime.
        As such, it's only possible to execute this method on entities generated from**
        ``MetadataEntity``'s **subclasses. Please, refer to their documentation of the** `merge` **method.**

        :param other: The entity which will be marked as to be deleted and whose properties will
         be merged into the current entity.
        :type other: MetadataEntity
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """

        # Here we must REDIRECT triples pointing
        # to 'other' to make them point to 'self':
        for res, entity in self.m_set.res_to_entity.items():
            triples_list: List[Tuple] = list(entity.g.triples((res, None, other.res)))
            for triple in triples_list:
                entity.g.remove(triple)
                new_triple = (triple[0], triple[1], self.res)
                entity.g.add(new_triple)

        types: List[URIRef] = other.get_types()
        for cur_type in types:
            self._create_type(cur_type)

        label: Optional[str] = other.get_label()
        if label is not None:
            self.create_label(label)

        self._was_merged = True
        self._merge_list = (*self._merge_list, other)

        # 'other' must be deleted AFTER the redirection of
        # triples pointing to it, since mark_as_to_be_deleted
        # also removes every triple pointing to 'other'
        other.mark_as_to_be_deleted()

    def commit_changes(self):
        self.preexisting_graph = Graph(identifier=self.g.identifier)
        if self._to_be_deleted:
            self.remove_every_triple()
        else:
            for triple in self.g.triples((self.res, None, None)):
                self.preexisting_graph.add(triple)
        self._to_be_deleted = False
        self._was_merged = False
        self._merge_list = tuple()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2022 Arcangelo Massari <arcangelo.massari@unibo.it>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
